/*
	PerlinizedMeshApp.as
	Lee Felarca
	http://www.zeropointnine.com/blog
	5-2007
	v0.9
		
	Source code licensed under a Creative Commons Attribution 3.0 License.
	http://creativecommons.org/licenses/by/3.0/
	Some Rights Reserved.
*/

package 
{
	import flash.display.Bitmap;
	import flash.display.Sprite;
	import flash.display.StageAlign;
	import flash.display.StageQuality;
	import flash.display.StageScaleMode;
	import flash.events.Event;
	import flash.events.KeyboardEvent;
	import flash.events.MouseEvent;
	import flash.text.TextField;
	import flash.text.TextFormat;
	
	import org.papervision3d.cameras.FreeCamera3D;
	import org.papervision3d.core.geom.renderables.Vertex3D;
	import org.papervision3d.core.layers.RenderLayer;
	import org.papervision3d.core.layers.utils.RenderLayerSortMode;
	import org.papervision3d.lights.PointLight3D;
	import org.papervision3d.materials.ColorMaterial;
	import org.papervision3d.materials.WireframeMaterial;
	import org.papervision3d.materials.special.CompositeMaterial;
	import org.papervision3d.materials.utils.MaterialsList;
	import org.papervision3d.objects.primitives.Cube;
	import org.papervision3d.render.BasicRenderEngine;
	import org.papervision3d.scenes.Scene3D;
	import org.papervision3d.view.Viewport3D;


	[SWF(width="600", height="600", frameRate="45", backgroundColor="#ffffff")]
	public class PerlinizedMeshApp extends Sprite 
	{
		private const DEGREE:Number = Math.PI/180;
		
		private const BITMAP_SIDE_LENGTH: Number = 112;
		
		private var _meshSideLength			: int = 50;
		private var _numOctaves				: int = 3;

		private var _scene3D				: Scene3D;
		private var _camera					: FreeCamera3D;
		private var _leeCamera				: LeeCamera3D;
		private var _renderer				: BasicRenderEngine;
		private var _viewport				: Viewport3D;
		private var _light					: PointLight3D;
		private var _stand					: Cube;
		private var _materials				: Array;
		private var _layerMesh				: RenderLayer;
		private var _layerStand				: RenderLayer;
		
		private var _perlinizedMesh			: PerlinizedMesh;
		
		private var _baseShapeNames			: Array = ["Sphere", "Dome", "Cylinder", "Bowl"];
		private var _baseShapeNum			: int;
		private var _radiusMultipliers		: Array = [0.0, 1.0, 1.5, 2.0];
		private var _radiusMultiplierIndex	: int = 3;
		private var _materialNames			: Array = ["Solid", "Wireframe", "Flat"];
		private var _materialIndex			: int;
		private var _morphMultipliers		: Array = [-4,-2,-1,-0.5,0,0.5,1,2,4];
		private var _morphMultiplierIndex	: int = 6;

		private var _tfShowHide					: TextField;
		private var _tfInfo					: TextField;
		private var _tfInfo2				: TextField;
		private var _tfPrompt				: TextField;
		private var _bitmap					: Bitmap;


		public function PerlinizedMeshApp()
		{
			this.stage.scaleMode = StageScaleMode.NO_SCALE;
			this.stage.align = StageAlign.TOP_LEFT;
			this.stage.quality = StageQuality.HIGH;

			init();
			makePerlinizedMesh();
		} 

		
		private function init():void
		{
			_scene3D = new Scene3D();
			
			_viewport = new Viewport3D(stage.stageWidth, stage.stageHeight);
			_layerStand = new RenderLayer();
			_layerStand.layerIndex = 1;
			_viewport.addRenderLayer(_layerStand);
			_layerMesh = new RenderLayer();
			_layerMesh.layerIndex = 1;
			_viewport.addRenderLayer(_layerMesh);
			this.addChild(_viewport);

			_light = new PointLight3D(true);
			_scene3D.addChild(_light);
			_camera = new FreeCamera3D();
			_camera.zoom = 7.5;
			_leeCamera = new LeeCamera3D( new Vertex3D(0,80,0), 300);

			_renderer = new BasicRenderEngine();
			
			var matList:MaterialsList = new MaterialsList();
			var m1:ColorMaterial = new ColorMaterial(0xccccdd,0.3);
			m1.doubleSided = true;
			var m2:ColorMaterial = new ColorMaterial(0xbbbbdd,0.3);
			m2.doubleSided = true;
			matList.addMaterial( m1, "all");
			matList.addMaterial( m2, "top");

			_stand = new Cube(matList,200,200,400,2,4,2);
			_stand.y = -200;
			_stand.renderLayer = _layerStand;
			_scene3D.addChild(_stand);

			this.stage.addEventListener(KeyboardEvent.KEY_DOWN, onKeyPress, false,0,true);
			this.stage.focus = this.stage;

			_tfShowHide = new TextField();
			_tfShowHide.defaultTextFormat = new TextFormat("Arial", 11, 0x888888, false,null,null,null,null,"right");
			_tfShowHide.width = 120;
			_tfShowHide.x = this.stage.stageWidth - _tfShowHide.width - 10;
			_tfShowHide.y = this.stage.stageHeight - 15 - 13;
			_tfShowHide.selectable = false;
			this.addChild(_tfShowHide);
			
			_tfInfo = new TextField();
			_tfInfo.defaultTextFormat = new TextFormat("Arial", 11, 0x888888, false);
			_tfInfo.width = 180;
			_tfInfo.multiline = true;
			_tfInfo.x = 10;
			_tfInfo.y = 10;
			_tfInfo.selectable = false;
			this.addChild(_tfInfo);

			_tfInfo2 = new TextField();
			_tfInfo2.defaultTextFormat = new TextFormat("Arial", 11, 0x888888, false);
			_tfInfo2.width = 180;
			_tfInfo2.multiline = true;
			_tfInfo2.x = 10;
			_tfInfo2.y = this.stage.stageHeight - 67 - 15;
			_tfInfo2.selectable = false;
			this.addChild(_tfInfo2);

			_bitmap = new Bitmap();
			_bitmap.visible = true;
			this.addChild(_bitmap);

			_materials = [];
			
			var matCol:ColorMaterial = new ColorMaterial(0x888899,0.5);
			var matWir:WireframeMaterial = new WireframeMaterial(0x000000);
			var matCom:CompositeMaterial = new CompositeMaterial();
			matCom.addMaterial( new ColorMaterial(0x2222) );
			matCom.addMaterial( new WireframeMaterial(0x999999));
			
			_materials.push(matCom);
			_materials.push(matWir);
			_materials.push(matCol);
			
			updateText();
			
			this.stage.addEventListener(MouseEvent.MOUSE_DOWN, onMouseDown, false,0,true);
			this.stage.addEventListener(Event.ENTER_FRAME, onEnterFrame, false,0,true);
		}
		
		
		private function makePerlinizedMesh():void
		{
			if (_perlinizedMesh) _perlinizedMesh.destroy();
			_scene3D.removeChildByName("mesh");

			var properties:Object = { 
				baseShape:_baseShapeNum,
				sizeMultiplier:100, 
				aspectRatio:2,
				radiusMultiplier:_radiusMultipliers[_radiusMultiplierIndex], 
				morphMultiplier:_morphMultipliers[_morphMultiplierIndex],
				perlinSeed:int(Math.random()*0xFFFFFF), 
				perlinOctaves:_numOctaves,
				perlinOctaveOffsetRandomnessRange:0.88
			};
			
			_perlinizedMesh = new PerlinizedMesh( _meshSideLength,_meshSideLength, _materials[_materialIndex], properties);
			_perlinizedMesh.triangleMesh3D.name = "mesh";

			_perlinizedMesh.triangleMesh3D.renderLayer = _layerMesh;

			_scene3D.addChild(_perlinizedMesh.triangleMesh3D);

			_viewport.renderLayerManager.sortMode = RenderLayerSortMode.INDEX_SORT;
			
			updateBitmap();
			updateText();
		}

		private function onKeyPress(e:KeyboardEvent):void
		{
			trace(e.charCode);
			switch (e.charCode)
			{
				case 45: 
					if (_meshSideLength > 10) {
						_meshSideLength -= 10;
						makePerlinizedMesh();
					}
				break;
				case 61: 
					if (_meshSideLength < 70) {
						_meshSideLength += 10;
						makePerlinizedMesh();
					}
				break;
				case 111:
					_numOctaves++;
					if (_numOctaves > 5) _numOctaves = 1;
					makePerlinizedMesh();
				break;
				case 98:
					_baseShapeNum++;
					if (_baseShapeNum >= _baseShapeNames.length) _baseShapeNum = 0;
					_perlinizedMesh.baseShape = _baseShapeNum;
				break;
				case 104:
					_bitmap.visible = !_bitmap.visible;
				break;
				case 100:
					_radiusMultiplierIndex++;
					if (_radiusMultiplierIndex >= _radiusMultipliers.length) _radiusMultiplierIndex = 0;
					_perlinizedMesh.radiusMultiplier = _radiusMultipliers[_radiusMultiplierIndex];
				break;
				case 109:
					_materialIndex++;
					if (_materialIndex >= _materials.length) _materialIndex = 0;
					_perlinizedMesh.material = _materials[_materialIndex];
				break;
				case 93:
					if (_morphMultiplierIndex < _morphMultipliers.length-1) {
						_morphMultiplierIndex++;
						_perlinizedMesh.morphMultiplier = _morphMultipliers[_morphMultiplierIndex];
					}					
				break;
				case 91:
					if (_morphMultiplierIndex > 0) {
						_morphMultiplierIndex--;
						_perlinizedMesh.morphMultiplier = _morphMultipliers[_morphMultiplierIndex];
					}					
				break;
			}
			
			updateText();
		}

		private function updateText():void
		{
			_tfInfo.text = 
				"[Click] Re-randomize\r" + 
				"[O] Octaves:  " + _numOctaves.toString() + "\r" +
				"[+,-] Vertices:  " + (_meshSideLength*_meshSideLength).toString();
			
			_tfShowHide.text = "[H] " + ((_bitmap.visible) ? "Hide source bitmap" : "Show source bitmap");
			
			_tfInfo2.text = 
				"[B] Base shape:  " + _baseShapeNames[ _baseShapeNum ] + "\r" +
				"[M] Material:  " + _materialNames[_materialIndex] + "\r" +
				"[D] Definition multiplier:  " + _radiusMultipliers[_radiusMultiplierIndex] + "\r" +
				"[Bracket keys] Morph speed:  " +  _morphMultipliers[_morphMultiplierIndex].toString(); 
		}
		
		private function updateBitmap():void
		{
			_bitmap.bitmapData = _perlinizedMesh.bitmapDataSource;
			_bitmap.scaleX = BITMAP_SIDE_LENGTH / _bitmap.bitmapData.width;
			_bitmap.scaleY = -BITMAP_SIDE_LENGTH /_bitmap.bitmapData.width;
			_bitmap.x = this.stage.stageWidth - (_meshSideLength * _bitmap.scaleX) - 15;
			_bitmap.y = _tfShowHide.y - 5;
			_bitmap.alpha = 0.60;
		}

		private function onEnterFrame(e:Event):void
		{
			_perlinizedMesh.updateMesh();

			var xfactor:Number = (stage.mouseX - stage.stageWidth/2) / (stage.stageWidth/2); 
			var yfactor:Number = -(stage.mouseY - stage.stageHeight/2) / (stage.stageHeight/2) * 1.2;
			if (yfactor > 1.1) yfactor = 1.1;
			if (yfactor < -0.33) yfactor = -0.33;
			
			_leeCamera.pitch = yfactor * 90;
			_leeCamera.yaw = 180 + xfactor * 90;
			_leeCamera.toFreeCamera3D(_camera);
			_light.x = _camera.x;
			_light.y = _camera.y;
			_light.z = _camera.z;

			// Manually managing z-depth. 14.4 degrees is the angle at which
			// the top of the stand is edge-on with the camera...
			_layerStand.layerIndex = (_leeCamera.pitch > -14.4) ? 2 : 0;
			
			_renderer.renderScene(_scene3D, _camera, _viewport, false);
			
		}
		
		private function onMouseDown(e:MouseEvent):void
		{
			makePerlinizedMesh();
		}
	}
}
